/*!
 * \file elapsedTimer.hpp
 * \brief This file is used to measure elapsed time.
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * [License]
 *
 * $id$
 */

#ifndef ELAPSED_TIMER_H
#define ELAPSED_TIMER_H

#include <sys/times.h>

#include "util.hpp"

/*!
 * \brief This class measure elapsed time by processed working.
 */
class TElapsedTimer {
  public:
    /*!
     * \brief TElapsedTimer's clock frequency.<br>
     *        Please set this value from external.<br>
     *        e.g. "sysconf(_SC_CLK_TCK)"
     */
    static long clock_ticks;
    
    /*!
     * \brief TElapsedTimer constructor.
     * \param label [in] Working process name.
     */
    TElapsedTimer(const char *label) {
      
      /* Stored process start time. */
      this->_start_clock = times(&this->_start_tms);
      this->_label = label;
    }
    
    /*!
     * \brief TElapsedTimer destructor.
     */
    ~TElapsedTimer() {
      
      /* Elapsed time is displayed when LogLevel is INFO only. */
      if (arg.LogLevel < INFO) {
        return;
      }
      
      /* Get process end time. */
      struct tms _end_tms;
      clock_t _end_clock = times(&_end_tms);
      
      /* Output message. */
      PRINT_INFO_MSG_HEADER_NOIF << "Elapsed Time";
      if (this->_label != NULL) {
        /* Output process name. */
        STDOUT << "(in " << this->_label << ")";
      }
      
      /* Calculate elapsed time and output result. */
      STDOUT
        << ": " << this->calcClockToSec(this->_start_clock, _end_clock)
        << " sec (user = "
        << this->calcClockToSec(this->_start_tms.tms_utime, _end_tms.tms_utime)
        << " sec, system = "
        << this->calcClockToSec(this->_start_tms.tms_stime, _end_tms.tms_stime)
        << " sec)" << NEWLINE;
    }
    
  private:
    /*!
     * \brief Working process name.
     */
    const char *_label;
    /*!
     * \brief Process start simple time.
     */
    clock_t _start_clock;
    /*!
     * \brief Process start detail time.
     */
    struct tms _start_tms;
    
    /*!
     * \brief Calculate time between start and end.
     * \param start [in] Time of process begin.
     * \param end   [in] Time of process finished.
     * \return Elapsed time.
     */
    inline double calcClockToSec(const clock_t start, const clock_t end) {
      return (double)(end - start) / (double)clock_ticks;
    }
};

#endif
