/*
 * BrowseButton.java
 * Created on 2012/01/24
 * RCSID: $Id: BrowseButton.java 385 2012-03-09 06:17:22Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */
package jp.co.ntt.oss.heapstats.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

/**
 * extends {@link AbstractCellEditor}<br>
 * implements {@link TableCellEditor}, {@link TableCellRenderer}<br>
 * <br>
 * Displayed in a text file data or contents of the specified line.
 */
public class BrowseButton extends AbstractCellEditor implements
                                                    TableCellEditor,
                                                    TableCellRenderer,
                                                    ActionListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = -4223033654188094768L;

    /** The name of a reference button. */
    private static final String BUTTON_NAME = "...";

    private static final int COL_IDX_PATH = 3;

    /** Table to display the details of the log. */
    private JTable itemEntries;

    /** Text area that displays the log information for the specified items. */
    private JTextArea detail;

    /** The reference button. */
    private JButton browse;

    /** LogDetail Data. */
    private LogDetailDialog detailDlg;

    /**
     * Create a Browse button to put on the table.
     *
     * @param parent Log Detail Data
     * @param table Table to display the details of the log
     * @param area Text area that displays the log information for the specified
     *        items
     */
    public BrowseButton(final LogDetailDialog parent, final JTable table,
            final JTextArea area) {
        detailDlg = parent;
        itemEntries = table;
        detail = area;

        browse = new JButton(BUTTON_NAME) {
            /** serialVersionUID. */
            private static final long serialVersionUID = -6292696235280698041L;

            @Override
            public void updateUI() {
                super.updateUI();
                setBorder(BorderFactory.createEmptyBorder());
                setFocusable(false);
                setRolloverEnabled(false);
                setText(BUTTON_NAME);
            }
        };

        browse.addActionListener(this);
    }

    @Override
    public final void addCellEditorListener(final CellEditorListener l) {
        listenerList.add(CellEditorListener.class, l);
    }

    @Override
    public final void cancelCellEditing() {
        fireEditingCanceled();
    }

    @Override
    public final Object getCellEditorValue() {
        return browse.getText();
    }

    @Override
    public final boolean isCellEditable(final EventObject l) {
        return true;
    }

    @Override
    public final void removeCellEditorListener(final CellEditorListener l) {
        listenerList.remove(CellEditorListener.class, l);
    }

    /**
     * Returns true if the editing cell should be selected, false otherwise.
     *
     * @param anEvent the event the editor should use to start editing
     * @return Returns false.
     */
    @Override
    public final boolean shouldSelectCell(final EventObject anEvent) {
        return false;
    }

    @Override
    public final boolean stopCellEditing() {
        fireEditingStopped();
        return true;
    }

    @Override
    public final Component getTableCellRendererComponent(final JTable table,
            final Object value, final boolean isSelected,
            final boolean hasFocus, final int row, final int column) {
        return browse;
    }

    @Override
    public final Component getTableCellEditorComponent(final JTable table,
            final Object value, final boolean isSelected, final int row,
            final int column) {
        return browse;
    }

    @Override
    public final void actionPerformed(final ActionEvent e) {
        int row = itemEntries.convertRowIndexToModel(itemEntries
                .getEditingRow());
        detail.setText("");
        detail.append(String.valueOf(itemEntries.getValueAt(row, 0)));
        detail.append("\n\n");

        Object path = itemEntries.getValueAt(row, COL_IDX_PATH);
        if (path == null || path.equals(BUTTON_NAME)) {
            detail.append(String.valueOf(itemEntries.getValueAt(row, 1)));
        } else {
            loadFile(path);
        }

        detail.setCaretPosition(0);

        fireEditingCanceled();
    }

    /**
     * Returns the contents of the file specified.
     *
     * @param path Absolute file path
     */
    private void loadFile(final Object path) {
        BufferedReader read = null;
        // StringBuilder sb = new StringBuilder();
        String[] pathList = String.valueOf(path).split(
                GuiCommon.SPLIT_CHARACTER);

        File file = null;
        try {
            for (String tmp : pathList) {
                file = new File(tmp);
                read = new BufferedReader(new FileReader(file));

                String line = read.readLine();

                while (line != null) {
                    // Converted to blank a null character.
                    int ind = line.indexOf("\0");
                    int prev = 0;
                    while (ind > 0 && ind < line.length()) {
                        detail.append(line.substring(prev, ind));
                        detail.append(" ");
                        prev = ind + 1;
                        ind = line.indexOf("\0", prev);
                    }

                    if (prev == 0) {
                        detail.append(line);
                    }

                    detail.append("\n");
                    line = read.readLine();
                }

                if (read != null) {
                    read.close();
                }
            }
        } catch (Throwable e) {
            if (file != null) {
                GuiCommon.showMessage(detailDlg, GuiCommon.BASEE08,
                        file.getName(), JOptionPane.ERROR_MESSAGE);
            } else {
                GuiCommon.showMessage(detailDlg, GuiCommon.BASEE08,
                        String.valueOf(path), JOptionPane.ERROR_MESSAGE);
            }
            e.printStackTrace();
        } finally {
            try {
                if (read != null) {
                    read.close();
                }
            } catch (IOException e) {
                GuiCommon.showMessage(detailDlg, GuiCommon.BASEE08,
                        file.getName(), JOptionPane.ERROR_MESSAGE);
                e.printStackTrace();
            }
        }
    }

}
