/*
 * HeapStatsParser.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.parser;

import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.co.ntt.oss.heapstats.container.ObjectData;
import jp.co.ntt.oss.heapstats.container.SnapShotHeader;
import jp.co.ntt.oss.heapstats.gui.GuiCommon;

/**
 * This class loads a file java heap information.
 */
public class HeapStatsParser {
    /**
     * Output to a temporary file and then parse the data in the snapshot agent
     * output.
     *
     * @param fname the file java heap information.
     * @param handler the ParserEventHandler.
     * @return Return the Reading results file
     * @throws IOException If you load the file java heap information is not set
     *         ByteOrder agent if that occurs
     */
    public final boolean parse(final String fname,
            final ParserEventHandler handler) throws IOException {
        SnapShotHeader header;
        FileInputStream stream = new FileInputStream(fname);

        try {
            while (true) {
                int ret = stream.read();

                if (ret == -1) {
                    // EOF
                    break;
                } else if (ret == '1') {
                    // Heap
                    header = parseHeader(stream);

                    if (handler.onNewSnapShot(header) != ParserEventHandler.ParseResult.HEAPSTATS_PARSE_CONTINUE)
                        return false;

                    switch (parseElement(stream, header, handler)) {
                    case HEAPSTATS_PARSE_ABORT:
                        return false;
                    case HEAPSTATS_PARSE_SKIP:
                        continue;
                    default:
                        break;
                    }
                } else {
                    StringBuilder errString = new StringBuilder();
                    errString.append("Unknown FileType! (");
                    errString.append(ret);
                    errString.append(")");
                    throw new IOException(errString.toString());
                }
            }
        } finally {
            stream.close();
        }

        return true;
    }

    /**
     * Extracting the header information of the snapshot.
     *
     * @param stream the file Java Heap Information.
     * @return Return the SnapShotHeader
     * @throws IOException If other I / O error occurs.
     */
    private SnapShotHeader parseHeader(final FileInputStream stream)
            throws IOException {
        SnapShotHeader header = new SnapShotHeader();
        int ret = stream.read();

        switch (ret) {
        case 'L':
            header.setByteOrderMark(ByteOrder.LITTLE_ENDIAN);
            break;
        case 'B':
            header.setByteOrderMark(ByteOrder.BIG_ENDIAN);
            break;
        default:
            StringBuilder errString = new StringBuilder();
            errString.append("Unknown ByteOrderMark! (");
            errString.append(ret);
            errString.append(")");
            throw new IOException(errString.toString());
        }

        FileChannel ch = stream.getChannel();
        ByteBuffer buf = ByteBuffer.allocateDirect(16);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // SnapShot Date
        header.setSnapShotDateAsLong(buf.getLong());
        // Entries
        header.setNumEntries(buf.getLong());

        buf = ByteBuffer.allocateDirect(4);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();
        // SnapShot Cause
        header.setCause(buf.getInt());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // GC Cause
        byte[] gcCause = new byte[(int) buf.getLong()];
        if (stream.read(gcCause) != gcCause.length) {
            throw new IOException("Could not get the GC Cause.");
        }

        if (gcCause[0] == '\0') {
            header.setGcCause("-");
        } else {
            header.setGcCause(new String(gcCause));
        }

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // Full GC Count
        header.setFullCount(buf.getLong());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // Young GC Count
        header.setYngCount(buf.getLong());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();
        // GC Time
        header.setGcTime(buf.getLong());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // New Heap Size
        header.setNewHeap(buf.getLong());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // Old Heap Size
        header.setOldHeap(buf.getLong());

        buf = ByteBuffer.allocateDirect(8);
        buf.order(header.getByteOrderMark());
        ch.read(buf);
        buf.flip();

        // Total Heap Size
        header.setTotalSize(buf.getLong());

        return header;
    }

    /**
     * Stored in a temporary file to extract the object information.
     *
     * @param stream the file Java Heap Information.
     * @param header the SnapShot header
     * @param handler ParserEventHandler
     * @return Return the Parse result.
     * @throws IOException If some other I/O error occurs
     */
    private ParserEventHandler.ParseResult parseElement(
            final FileInputStream stream, final SnapShotHeader header,
            final ParserEventHandler handler) throws IOException {

        ByteBuffer buf = ByteBuffer.allocateDirect(16);
        buf.order(header.getByteOrderMark());

        byte[] classNameInBytes;
        ParserEventHandler.ParseResult eventResult;
        ObjectData obj;
        FileChannel ch = stream.getChannel();

        for (long i = 0; i < header.getNumEntries(); i++) {
            obj = new ObjectData();
            buf.clear();
            ch.read(buf);
            buf.flip();

            // tag
            obj.setTag(buf.getLong());

            // class Name
            classNameInBytes = new byte[(int) buf.getLong()];
            if (stream.read(classNameInBytes) != classNameInBytes.length) {
                throw new IOException("Could not get the Class name.");
            }

            if (GuiCommon.getClassRename()) {
                String tmp = new String(classNameInBytes).replaceAll(
                        "^L|(^\\[*)L|;$", "$1");
                tmp = tmp.replaceAll("(^\\[*)B$", "$1byte");
                tmp = tmp.replaceAll("(^\\[*)C$", "$1char");
                tmp = tmp.replaceAll("(^\\[*)I$", "$1int");
                tmp = tmp.replaceAll("(^\\[*)S$", "$1short");
                tmp = tmp.replaceAll("(^\\[*)J$", "$1long");
                tmp = tmp.replaceAll("(^\\[*)D$", "$1double");
                tmp = tmp.replaceAll("(^\\[*)F$", "$1float");
                tmp = tmp.replaceAll("(^\\[*)V$", "$1void");
                tmp = tmp.replaceAll("(^\\[*)Z$", "$1boolean");

                Pattern pattern = Pattern.compile("^\\[(.*)");
                Matcher matcher = pattern.matcher(tmp);
                while (matcher.find()) {
                    tmp = matcher.replaceAll("$1 []");
                    matcher = pattern.matcher(tmp);
                }

                obj.setName(tmp.replaceAll("\\/", "."));
            } else {
                obj.setName(new String(classNameInBytes));
            }

            buf.clear();
            ch.read(buf);
            buf.flip();

            // instance
            obj.setCount(buf.getLong());
            // heap usage
            obj.setTotalSize(buf.getLong());
            eventResult = handler.onEntry(obj);

            if (eventResult != ParserEventHandler.ParseResult.HEAPSTATS_PARSE_CONTINUE) {
                return eventResult;
            }
        }

        return ParserEventHandler.ParseResult.HEAPSTATS_PARSE_CONTINUE;
    }

}
